@@ -3,6 +3,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 use cow_rc_str::CowRcStr;
+use smallvec::SmallVec;
 use std::ops::Range;
 use std::ascii::AsciiExt;
 use std::ops::BitOr;
@@ -101,6 +102,15 @@ impl<'i> ParserInput<'i> {
         }
     }
 
+    /// Create a new input for a parser.  Line numbers in locations
+    /// are offset by the given value.
+    pub fn new_with_line_number_offset(input: &'i str, first_line_number: u32) -> ParserInput<'i> {
+        ParserInput {
+            tokenizer: Tokenizer::with_first_line_number(input, first_line_number),
+            cached_token: None,
+        }
+    }
+
     #[inline]
     fn cached_token_ref(&self) -> &Token<'i> {
         &self.cached_token.as_ref().unwrap().token
@@ -291,10 +301,8 @@ impl<'i: 't, 't> Parser<'i, 't> {
     /// Advance the input until the next token that’s not whitespace or a comment.
     #[inline]
     pub fn skip_whitespace(&mut self) {
-        // If we just consumed `{`, `[`, `(`, or `function(`, leave whitespace
-        // or comments inside the block or function up to the nested parser.
-        if self.at_start_of.is_some() {
-            return
+        if let Some(block_type) = self.at_start_of.take() {
+            consume_until_end_of_block(block_type, &mut self.input.tokenizer);
         }
 
         self.input.tokenizer.skip_whitespace()
@@ -302,6 +310,10 @@ impl<'i: 't, 't> Parser<'i, 't> {
 
     #[inline]
     pub(crate) fn skip_cdc_and_cdo(&mut self) {
+        if let Some(block_type) = self.at_start_of.take() {
+            consume_until_end_of_block(block_type, &mut self.input.tokenizer);
+        }
+
         self.input.tokenizer.skip_cdc_and_cdo()
     }
 
@@ -849,7 +861,8 @@ pub fn parse_nested_block<'i: 't, 't, F, T, E>(parser: &mut Parser<'i, 't>, pars
 }
 
 fn consume_until_end_of_block(block_type: BlockType, tokenizer: &mut Tokenizer) {
-    let mut stack = vec![block_type];
+    let mut stack = SmallVec::<[BlockType; 16]>::new();
+    stack.push(block_type);
 
     // FIXME: have a special-purpose tokenizer method for this that does less work.
     while let Ok(ref token) = tokenizer.next() {
